
# q-expansion of newform 2520.2.a.h, downloaded from the LMFDB on 05 May 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 2520
weight = 2
poly_data = [0, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = None
aps_data = [[0], [0], [-1], [1], [2], [-2], [-2], [-8], [2], [8], [-8], [-6], [-6], [4], [-4], [0], [4], [-2], [-4], [-6], [-6], [0], [16], [-6], [-14], [-18], [4], [6], [2], [-12], [0], [0], [-16], [-12], [-20], [-16], [10], [-20], [12], [22], [-2], [2], [10], [-2], [8], [20], [20], [-16], [-12], [-14], [0], [30], [14], [-12], [-6], [18], [10], [-24], [-10], [-28], [20], [14], [4], [4], [-10], [-24], [-12], [22], [-14], [6], [22], [-6], [8], [22], [28], [24], [24], [-14], [-36], [-2], [-12], [30], [-14], [14], [4], [42], [-24], [-26], [6], [24], [28], [-4], [16], [-18], [20], [0], [2], [14], [-8], [-26], [44], [16], [-4], [-20], [4], [38], [12], [6], [-30], [-14], [0], [30], [0], [-4], [16], [-12], [-12], [-12], [4], [-30], [30], [30], [38], [-6], [-28], [-8], [-22], [32], [-4], [-42], [-12], [30], [24], [-46], [-30], [-14], [54], [8], [-18], [-24], [-28], [20], [-8], [6], [34], [28], [-46], [-6], [32], [-6], [22], [-42], [-4], [-52], [36], [26], [-16], [50], [-50], [50], [-58], [60], [48], [-56], [-32], [24], [8], [-30], [-22], [22], [10], [22], [10], [-22], [20], [30], [-44], [40], [-20], [-50], [-64], [0], [38], [62], [-14], [22], [-46], [-16], [-22], [-40], [2], [-18], [-52], [-2], [-58], [-50], [30], [-66], [-38], [-44], [-36], [-44], [14], [-66], [0], [70], [-20], [38], [0], [-16], [-38], [-62], [8], [48], [-28], [18], [16], [22], [-6], [-4], [26], [16], [-40], [-24], [72], [58], [-14], [-50], [-28], [70], [10], [24], [-16], [56], [60], [0], [-66], [-24], [14], [-48], [-46], [4], [-32], [10], [34], [-44], [-44], [4], [28], [34], [-78], [-34], [6], [14], [22], [18], [-10], [-60], [54], [-18], [-56], [10], [-6], [-70], [-2], [-44], [-4], [30], [12], [-4], [-10], [52], [18], [56], [-46], [68], [38], [74], [14], [-60], [-24], [-48], [-56], [-26], [-8], [74], [14], [24], [48], [-30], [44], [-72], [-48], [60], [-50], [6], [44], [-30], [-28], [0], [54], [-60], [-64], [-50], [-60], [70], [-54], [34], [18], [-70], [56], [24], [84], [52], [82], [-74], [80], [54], [10], [20], [80], [86], [30], [-8], [-52], [30], [4], [-52], [-2], [-84], [50], [-76], [16], [-76], [28], [36], [62], [18], [70], [60], [-10], [-72], [-70], [-64], [44], [-22], [10], [28], [72], [78], [64], [-54], [-28], [-16], [-10], [-58], [0], [8], [44], [-2], [-10], [-94], [70], [-6], [-48], [22], [42], [56], [-34], [-42], [56], [46], [60], [-48], [-34], [0], [-2], [-10], [6], [2], [22], [-28], [-48], [36], [36], [-88], [88], [-18], [-36], [68], [-62], [-6], [42], [-20], [58], [62], [20], [-66], [-4], [24], [30], [-36], [-32], [42], [-30], [-36], [-2], [-48], [-56], [84], [22], [-60], [-62], [28], [-38], [-54], [-82], [-20], [6], [-84], [0], [58], [62], [32], [78], [-90], [-78], [84], [-28], [52]]
